/*
 * MentaAffinity => http://mentaaffinity.soliveirajr.com
 * Copyright (C) 2012  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentaaffinity;

/**
 * Any kind of affinity exception related to MentaAffinity.
 * 
 * @author Sergio Oliveira Jr.
 */
public class AffinityException extends RuntimeException {
	protected final Throwable rootCause;

	public AffinityException() {
		super();
		this.rootCause = null;
	}

	public AffinityException(final Throwable e) {
		super(getMsg(e), e);
		final Throwable root = getRootCause(e);
		this.setStackTrace(root.getStackTrace());
		this.rootCause = root == this ? null : root;
	}

	public AffinityException(final String msg) {
		super(msg);
		this.rootCause = null;
	}

	public AffinityException(final String msg, final Throwable e) {
		super(msg, e);
		final Throwable root = getRootCause(e);
		this.setStackTrace(root.getStackTrace());
		this.rootCause = root == this ? null : root;
	}

	private static String getMsg(final Throwable t) {
		final Throwable root = getRootCause(t);
		String msg = root.getMessage();
		if (msg == null || msg.length() == 0) {
			msg = t.getMessage();
			if (msg == null || msg.length() == 0) {
				return root.getClass().getName();
			}
		}
		return msg;
	}

	private static Throwable getRootCause(final Throwable t) {
		Throwable root = t.getCause();
		if (root == null) {
			return t;
		}
		while(root.getCause() != null) {
			root = root.getCause();
		}
		return root;
	}

	@Override
	public Throwable getCause() {
		return rootCause;
	}
}
