/*
 * MentaAffinity => http://mentaaffinity.soliveirajr.com
 * Copyright (C) 2012  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentaaffinity.sample;

import org.mentaaffinity.Affinity;

public class BasicBindToProcessor {

	public static void main(String[] ags) throws Exception {

		// first you have to mark all threads that you want to control the affinity
		// this must be done inside the thread, in other words, inside the run method

		Thread thread = new Thread(new Runnable() {
			@Override
			public void run() {
				Affinity.bind();
				try {
					while(true) {
						// do whatever you have to do here...
					}
				} finally {
					Affinity.unbind();
				}
			}
		}, "MyHungryThread");

		// after all your threads are marked, then you set the affinity however you want
		// of course you must set the affinity BEFORE the bind method above is called

		Affinity.assignToAnyFreeProcessor(thread);
		Affinity.printSituation();

		// OUTPUT:
		//		Chip-0:
		//		    Core-0.0:
		//		        Processor-0: assigned to MyHungryThread (not-started)
		//		    Core-1.0:
		//		        Processor-1: free
		//		    Core-2.0:
		//		        Processor-2: free
		//		    Core-8.0:
		//		        Processor-3: free
		//		    Core-9.0:
		//		        Processor-4: free
		//		    Core-10.0:
		//		        Processor-5: free

		// or

		Affinity.assignToProcessor(2, thread); // assign to processor 2
		Affinity.printSituation();

		// OUTPUT:
		//		Chip-0:
		//		    Core-0.0:
		//		        Processor-0: free
		//		    Core-1.0:
		//		        Processor-1: free
		//		    Core-2.0:
		//		        Processor-2: assigned to MyHungryThread (not-started)
		//		    Core-8.0:
		//		        Processor-3: free
		//		    Core-9.0:
		//		        Processor-4: free
		//		    Core-10.0:
		//		        Processor-5: free

		// now start your thread and be happy:
		thread.start();
		Affinity.printSituation();

		// OUTPUT:
		//		Chip-0:
		//		    Core-0.0:
		//		        Processor-0: free
		//		    Core-1.0:
		//		        Processor-1: free
		//		    Core-2.0:
		//		        Processor-2: bound to MyHungryThread (running)
		//		    Core-8.0:
		//		        Processor-3: free
		//		    Core-9.0:
		//		        Processor-4: free
		//		    Core-10.0:
		//		        Processor-5: free
	}
}
