/*
 * MentaAffinity => http://mentaaffinity.soliveirajr.com
 * Copyright (C) 2012  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentaaffinity.sample;

import org.mentaaffinity.Affinity;

/**
 * If you have a machine with more than one physical processor, you may want to make sure each thread land on a separate physical processor.
 * 
 * @author Sergio Oliveira Jr.
 */
public class BindToDifferentChips {

	public static void main(String[] ags) throws Exception {

		// first you have to mark all threads that you want to control the affinity
		// this must be done inside the thread, in other words, inside the run method

		Thread thread1 = new Thread(new Runnable() {
			@Override
			public void run() {
				Affinity.bind();
				try {
					while(true) {
						// do whatever you have to do here...
					}
				} finally {
					Affinity.unbind();
				}
			}
		}, "MyHungryThread1");

		Thread thread2 = new Thread(new Runnable() {
			@Override
			public void run() {
				Affinity.bind();
				try {
					while(true) {
						// do whatever you have to do here...
					}
				} finally {
					Affinity.unbind();
				}
			}
		}, "MyHungryThread2");

		// after all your threads are marked, then you set the affinity however you want
		// of course you must set the affinity BEFORE the bind method above is called

		Affinity.assignToDifferentChips(thread1, thread2); // you can also pass two chipIds to choose the chip...
		Affinity.printSituation();

		// OUTPUT:
		//		Chip-0:
		//		    Core-0.0:
		//		        Processor-0: assigned to MyHungryThread1 (not-started)
		//		        Processor-1: free
		//		    Core-1.0:
		//		        Processor-2: free
		//		        Processor-3: free
		//		Chip-1:
		//		    Core-0.1:
		//		        Processor-4: free
		//		        Processor-5: free
		//		    Core-1.1:
		//		        Processor-6: assigned to MyHungryThread2 (not-started)
		//		        Processor-7: free

		thread1.start();
		thread2.start();
		Affinity.printSituation();

		// OUTPUT:
		//		Chip-0:
		//		    Core-0.0:
		//		        Processor-0: bound to MyHungryThread1 (running)
		//		        Processor-1: free
		//		    Core-1.0:
		//		        Processor-2: free
		//		        Processor-3: free
		//		Chip-1:
		//		    Core-0.1:
		//		        Processor-4: free
		//		        Processor-5: free
		//		    Core-1.1:
		//		        Processor-6: bound to MyHungryThread2 (running)
		//		        Processor-7: free

	}
}
